package main;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.LinkedList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.awt.Color;

import template.CuttingTemplate;
import template.Line;

@SuppressWarnings("serial")
public class MainFrame extends JFrame {

	private static final Dimension frameSize = new Dimension(900, 500);
	private static final String title = "H10";

	private JPanel imagePanel;
	private JLabel labelCurrentFile;
	private JButton buttonPrevious;
	private JButton buttonNext;

	protected Function<Line, Color> currentColorFct;

	protected LinkedList<CuttingTemplate> templates = new LinkedList<CuttingTemplate>();
	protected int visibleTemplateIndex = 0;
	protected Image currentImage = null;

	public static String[] readAllLines(String filePath) throws IOException {
		// H1
		/*
		Stream<String> stream = Files.lines(Paths.get(filePath));
		String[] lines = stream.filter(s -> !s.isEmpty()).toArray(String[]::new);
		return lines;
		*/
		// One Liner!
		return Files.lines(Paths.get(filePath)).filter(s -> !s.isEmpty()).toArray(String[]::new);
	}

	/**
	 * Loads all cutting templates in the folder "templates"
	 */
	private void loadAllCuttingTemplates() {
		// H4
		try {
			// Create Variable "files" as Stream<Path> (because it include file paths)
			Stream<Path> files;
			// Get all files in templates (Attention! Directory "templates" is included!)
			files = Files.walk(Paths.get("templates"));
			// Convert files to List with Type Path to handle the files
			List<Path> filesList = files.collect(Collectors.toList());
			
			// Loop the files in filesList
			for (int i = 0; i < filesList.size(); i++) {
				// Get the path of file from filesList
				Path file = filesList.get(i);
				// Convert Path to String (fileName as String)
				String fileName = file.toString();
				
				// Check if fileName contains ".txt", if not its not correct file type
				// Attention! "templates" is a directory and do not include ".txt"
				// so you ignore the directory with this check - and this is good :-)
				if (fileName.contains(".txt")) {
					// If a file cannot read, it throw an error
					try {
						// Read lines from file
						String[] lines = readAllLines(fileName);
						// Create CuttingTemplate with the information from file (lines)
						CuttingTemplate ct = CuttingTemplate.parseCuttingTemplate(lines);
						// Set the name of template by fileName
						ct.setName(fileName);
						
						// Add CuttingTemplate to templates list
						this.templates.add(ct);
					} catch (IOException e) {
						// Error: File cannot read
						System.err.println("Unable to load file: " + fileName);
					}
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unused")
	public static void main(String[] args) {
		MainFrame mf = new MainFrame();
	}

	/**
	 * Change the current visible cutting template
	 * 
	 * @param i
	 */
	private void changeCurrentCuttingTemplate(int i) {
		int newIndex = visibleTemplateIndex + i;
		if (newIndex > templates.size() - 1) {
			newIndex = 0;
		}

		if (newIndex < 0) {
			newIndex = templates.size() - 1;
		}

		visibleTemplateIndex = newIndex;

		labelCurrentFile.setText(templates.get(visibleTemplateIndex).getName() + " (" + visibleTemplateIndex + ")");

		drawCurrentCuttingTemplate();
	}

	/**
	 * Redraws the current visible cutting template
	 */
	protected void drawCurrentCuttingTemplate() {
		currentImage = templates.get(visibleTemplateIndex).drawTemplate(currentColorFct);
		imagePanel.setSize(templates.get(visibleTemplateIndex).getWidth(),
				templates.get(visibleTemplateIndex).getHeight());
		imagePanel.repaint();
	}

	/**
	 * MainFrame constructor
	 */
	public MainFrame() {
		super(title);
		setSize(frameSize);
		getContentPane().setPreferredSize(frameSize);
		setLayout(new BorderLayout());

		imagePanel = new JPanel() {
			@Override
			protected void paintComponent(Graphics g) {
				super.paintComponent(g);
				if (currentImage != null) {
					g.drawImage(currentImage, 0, 0, null);
				}
			}
		};
		add(imagePanel, BorderLayout.CENTER);

		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new FlowLayout()); // default layout

		buttonPrevious = new JButton("<-");

		buttonPrevious.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeCurrentCuttingTemplate(-1);
				labelCurrentFile
						.setText(templates.get(visibleTemplateIndex).getName() + " (" + visibleTemplateIndex + ")");
			}
		});

		labelCurrentFile = new JLabel("no template selected");
		buttonNext = new JButton("->");

		buttonNext.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeCurrentCuttingTemplate(1);
			}
		});

		controlPanel.add(buttonPrevious);
		controlPanel.add(labelCurrentFile);
		controlPanel.add(buttonNext);

		add(controlPanel, BorderLayout.PAGE_END);

		currentColorFct = (Line l) -> {
			return Color.BLUE;
		};

		loadAllCuttingTemplates();
		if (templates.size() > 0) {
			changeCurrentCuttingTemplate(0);
		}

		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
	}
}
